@@ -19,8 +19,6 @@
 #include "playlistexporter.h"
 #include "playlistitem.h"
 
-class QWidget;
-class DCOPRef;
 class PlaylistBox;
 class PlaylistAction;
 
@@ -49,15 +47,17 @@ protected:
 
 private slots:
     void slotExport();
-    
+
 private:
     enum K3bOpenMode { AudioCD, DataCD, Abort };
 
     // Private method declarations
     void exportViaCmdLine(const PlaylistItemList &items);
+#if 0
     void exportViaDCOP(const PlaylistItemList &items, DCOPRef &ref);
     void DCOPErrorMessage();
     bool startNewK3bProject(DCOPRef &ref);
+#endif
     K3bOpenMode openMode();
 
     // Private member variable declarations
@@ -85,10 +85,10 @@ public:
 private slots:
     void slotExport();
 
-private:    
+private:
     PlaylistBox *m_playlistBox;
 };
 
 #endif /* K3BEXPORTER_H */
 
-// vim: set et ts=4 sw=4:
+// vim: set et sw=4 tw=0 sta:
