@@ -25,7 +25,9 @@
 // include files for QT
 #include <QCheckBox>
 #include <QLabel>
+#include <QDesktopWidget>
 #include <qradiobutton.h>
+#include <QCursor>
 #include <KTabWidget>
 
 // include files for KDE
@@ -56,21 +58,23 @@
 #include "kmixtoolbox.h"
 #include "version.h"
 #include "viewdockareapopup.h"
-//#include "osd.h" // Postponed to KDE4.1
+#include "dialogselectmaster.h"
+
+#include "osdwidget.h"
 
 
 /* KMixWindow
  * Constructs a mixer window (KMix main window)
  */
-KMixWindow::KMixWindow()
-   : KXmlGuiWindow(0),
+KMixWindow::KMixWindow(bool invisible)
+   : KXmlGuiWindow(0, Qt::WindowFlags( KDE_DEFAULT_WINDOWFLAGS | Qt::WindowContextHelpButtonHint) ),
    m_showTicks( true ),
-   m_showMenubar(true),
 //   m_isVisible (false),    // initialize, as we don't trigger a hideEvent()
 //   m_visibilityUpdateAllowed( true ),
    m_multiDriverMode (false), // -<- I never-ever want the multi-drivermode to be activated by accident
-   m_dockWidget()
-   , _dockAreaPopup(0)
+   m_dockWidget(),
+   m_dontSetDefaultCardOnStart (false),
+   _dockAreaPopup(0)
 {
     setObjectName("KMixWindow");
     // disable delete-on-close because KMix might just sit in the background waiting for cards to be plugged in
@@ -78,16 +82,17 @@ KMixWindow::KMixWindow()
 
    initActions(); // init actions first, so we can use them in the loadConfig() already
    loadConfig(); // Load config before initMixer(), e.g. due to "MultiDriver" keyword
+   KGlobal::locale()->insertCatalog("kmix-controls");
    initWidgets();
    initPrefDlg();
    MixerToolBox::instance()->initMixer(m_multiDriverMode, m_hwInfoString);
    KMixDeviceManager *theKMixDeviceManager = KMixDeviceManager::instance();
-   recreateGUI();
+   recreateGUI(false);
    fixConfigAfterRead();
    theKMixDeviceManager->initHotplug();
    connect(theKMixDeviceManager, SIGNAL( plugged( const char*, const QString&, QString&)), SLOT (plugged( const char*, const QString&, QString&) ) );
    connect(theKMixDeviceManager, SIGNAL( unplugged( const QString&)), SLOT (unplugged( const QString&) ) );
-   if ( m_startVisible )
+   if ( m_startVisible && ! invisible)
       show(); // Started visible: We don't do "m_isVisible = true;", as the showEvent() already does it
 
    connect( kapp, SIGNAL( aboutToQuit()), SLOT( saveConfig()) );
@@ -112,7 +117,7 @@ void KMixWindow::initActions()
    KStandardAction::preferences( this, SLOT(showSettings()), actionCollection());
    KStandardAction::keyBindings( guiFactory(), SLOT(configureShortcuts()), actionCollection());
 
-   QAction *action = actionCollection()->addAction( "hwinfo" );
+   KAction *action = actionCollection()->addAction( "hwinfo" );
    action->setText( i18n( "Hardware &Information" ) );
    connect(action, SIGNAL(triggered(bool) ), SLOT( slotHWInfo() ));
    action = actionCollection()->addAction( "hide_kmixwindow" );
@@ -122,6 +127,27 @@ void KMixWindow::initActions()
    action = actionCollection()->addAction("toggle_channels_currentview");
    action->setText(i18n("Configure &Channels..."));
    connect(action, SIGNAL(triggered(bool) ), SLOT(slotConfigureCurrentView()));
+   action = actionCollection()->addAction( "select_master" );
+   action->setText( i18n("Select Master Channel...") );
+   connect(action, SIGNAL(triggered(bool) ), SLOT(slotSelectMaster()));
+
+   KAction* globalAction = actionCollection()->addAction("increase_volume");
+   globalAction->setText(i18n("Increase Volume"));
+   globalAction->setGlobalShortcut(KShortcut(Qt::Key_VolumeUp));
+   connect(globalAction, SIGNAL(triggered(bool) ), SLOT(slotIncreaseVolume()));
+
+   globalAction = actionCollection()->addAction("decrease_volume");
+   globalAction->setText(i18n("Decrease Volume"));
+   globalAction->setGlobalShortcut(KShortcut(Qt::Key_VolumeDown));
+   connect(globalAction, SIGNAL(triggered(bool) ), SLOT(slotDecreaseVolume()));
+
+   globalAction = actionCollection()->addAction("mute");
+   globalAction->setText(i18n("Mute"));
+   globalAction->setGlobalShortcut(KShortcut(Qt::Key_VolumeMute));
+   connect(globalAction, SIGNAL(triggered(bool) ), SLOT(slotMute()));
+
+   osdWidget = new OSDWidget();
+
    createGUI( "kmixui.rc" );
 }
 
@@ -183,17 +209,23 @@ bool KMixWindow::updateDocking()
       return false;
    }
 
-    // create dock widget and the corresponding popup
-    /* A GUIProfile does not make sense for the DockAreaPopup => Using (GUIProfile*)0 */
+   // create dock widget and the corresponding popup
+   /* A GUIProfile does not make sense for the DockAreaPopup => Using (GUIProfile*)0 */
    QWidget* referenceWidgetForSystray = this;
-     if ( m_volumeWidget ) {
-      _dockAreaPopup = new ViewDockAreaPopup(0, "dockArea", Mixer::getGlobalMasterMixer(), 0, (GUIProfile*)0, this);
+   if ( m_volumeWidget ) {
+      KMenu *volMenu = new KMenu(this);
+      _dockAreaPopup = new ViewDockAreaPopup(volMenu, "dockArea", Mixer::getGlobalMasterMixer(), 0, (GUIProfile*)0, this);
       _dockAreaPopup->createDeviceWidgets();
-      referenceWidgetForSystray = _dockAreaPopup;
-    }
-    m_dockWidget = new KMixDockWidget( this, referenceWidgetForSystray, _dockAreaPopup );
-    m_dockWidget->show();
-    return true;
+
+      QWidgetAction *volWA = new QWidgetAction(volMenu);
+      volWA->setDefaultWidget(_dockAreaPopup);
+      volMenu->addAction(volWA);
+      referenceWidgetForSystray = volMenu;
+   }
+   m_dockWidget = new KMixDockWidget( this, referenceWidgetForSystray, _dockAreaPopup );
+   //m_dockWidget->show();
+   connect(m_dockWidget, SIGNAL(newMasterSelected()), SLOT(saveConfig()) );
+   return true;
 }
 
 void KMixWindow::saveConfig()
@@ -235,7 +267,7 @@ void KMixWindow::saveBaseConfig()
    }
    QString mixerIgnoreExpression = MixerToolBox::instance()->mixerIgnoreExpression();
    config.writeEntry( "MixerIgnoreExpression", mixerIgnoreExpression );
-   
+
    // @todo basically this should be moved in the views later (e.g. KDE4.2 ?)
    if ( m_toplevelOrientation  == Qt::Horizontal )
       config.writeEntry( "Orientation","Horizontal" );
@@ -251,7 +283,7 @@ void KMixWindow::saveViewConfig()
         QWidget *w = m_wsMixers->widget(i);
         if ( w->inherits("KMixerWidget") ) {
             KMixerWidget* mw = (KMixerWidget*)w;
-            // Here also Views are saved. even for Mixers that are closed. This is neccesary when unplugging cards.
+            // Here also Views are saved. even for Mixers that are closed. This is necessary when unplugging cards.
             // Otherwise the user will be confused afer re-plugging the card (as the config was not saved).
             mw->saveConfig( KGlobal::config().data() );
         }
@@ -355,10 +387,16 @@ KMixWindow::loadVolumes()
 
 
 
+void KMixWindow::recreateGUIwithoutSavingView()
+{
+	recreateGUI(false);
+}
+
+
 /**
  * Create or recreate the Mixer GUI elements
  */
-void KMixWindow::recreateGUI()
+void KMixWindow::recreateGUI(bool saveConfig)
 {
    saveViewConfig();  // save the state before recreating
    clearMixerWidgets();
@@ -410,7 +448,7 @@ void KMixWindow::plugged( const char* driverName, const QString& /*udi*/, QStrin
     if ( mixer != 0 ) {
         kDebug(67100) << "Plugged: dev=" << dev << "\n";
         MixerToolBox::instance()->possiblyAddMixer(mixer);
-        recreateGUI();
+        recreateGUI(true);
     }
 
 // Test code for OSD. But OSD is postponed to KDE4.1
@@ -442,7 +480,7 @@ void KMixWindow::unplugged( const QString& udi)
                 }
             }
             MixerToolBox::instance()->removeMixer(mixer);
-            // Check whether the Global Master disappeared, and select a new one if neccesary
+            // Check whether the Global Master disappeared, and select a new one if necessary
             MixDevice* md = Mixer::getGlobalMasterMD();
             if ( globalMasterMixerDestroyed || md == 0 ) {
                 // We don't know what the global master should be now.
@@ -452,7 +490,7 @@ void KMixWindow::unplugged( const QString& udi)
                     Mixer::setGlobalMaster( ((Mixer::mixers())[0])->id(), localMaster);
                     
                     QString text;
-                    text = i18n("The soundcard containing the master device was unplugged. Changing to control %1 on card %2", 
+                    text = i18n("The soundcard containing the master device was unplugged. Changing to control %1 on card %2.", 
                             ((Mixer::mixers())[0])->getLocalMasterMD()->readableName(),
                             ((Mixer::mixers())[0])->readableName()
                                 );
@@ -464,7 +502,7 @@ void KMixWindow::unplugged( const QString& udi)
                 text = i18n("The last soundcard was unplugged.");
                 KMixToolBox::notification("MasterFallback", text);
             }
-            recreateGUI();
+            recreateGUI(true);
             break;
         }
     }
@@ -506,7 +544,7 @@ void KMixWindow::addMixerWidget(const QString& mixer_ID)
    {
 //       kDebug(67100) << "KMixWindow::addMixerWidget() " << mixer_ID << " is being added";
       ViewBase::ViewFlags vflags = ViewBase::HasMenuBar;
-      if ( m_showMenubar ) {
+      if ( (_actionShowMenubar==0) || _actionShowMenubar->isChecked() ) {
             vflags |= ViewBase::MenuBarVisible;
       }
       if ( m_toplevelOrientation == Qt::Vertical ) {
@@ -552,7 +590,9 @@ bool KMixWindow::queryClose ( )
         return false;
     }
     else {
-        // Accept the close in all situations, when the user has disabled docking
+        // Accept the close, if:
+        //     The user has disabled docking
+        // or  SessionSaving() is running
 //         kDebug(67100) << "close";
         return true;
     }
@@ -570,6 +610,62 @@ void KMixWindow::hideOrClose ( )
     }
 }
 
+// internal helper to prevent code duplication in slotIncreaseVolume and slotDecreaseVolume
+void KMixWindow::increaseOrDecreaseVolume(bool increase)
+{
+  Mixer* mixer = Mixer::getGlobalMasterMixer(); // only needed for the awkward construct below
+  if ( mixer == 0 ) return; // e.g. when no soundcard is available
+  MixDevice *md = Mixer::getGlobalMasterMD();
+  if ( md == 0 ) return; // shouldn't happen, but lets play safe
+  md->setMuted(false);
+  if (increase)
+    mixer->increaseVolume(md->id());    // this is awkward. Better move the increaseVolume impl to the Volume class.
+  else
+    mixer->decreaseVolume(md->id());
+  // md->playbackVolume().increase(); // not yet implemented
+  showVolumeDisplay();
+}
+
+void KMixWindow::slotIncreaseVolume()
+{
+  increaseOrDecreaseVolume(true);
+}
+
+void KMixWindow::slotDecreaseVolume()
+{
+  increaseOrDecreaseVolume(false);
+}
+
+void KMixWindow::showVolumeDisplay()
+{
+  Mixer* mixer = Mixer::getGlobalMasterMixer();
+  if ( mixer == 0 ) return; // e.g. when no soundcard is available
+  MixDevice *md = Mixer::getGlobalMasterMD();
+  if ( md == 0 ) return; // shouldn't happen, but lets play safe
+  int currentVolume = mixer->volume(md->id());
+  
+  osdWidget->setCurrentVolume(currentVolume, md->isMuted());
+  osdWidget->show();
+  osdWidget->activateOSD(); //Enable the hide timer
+
+  //Center the OSD
+  QRect rect = KApplication::kApplication()->desktop()->screenGeometry(QCursor::pos());
+  QSize size = osdWidget->sizeHint();
+  int posX = rect.x() + (rect.width() - size.width()) / 2;
+  int posY = rect.y() + 4 * rect.height() / 5;
+  osdWidget->setGeometry(posX, posY, size.width(), size.height());
+}
+
+void KMixWindow::slotMute()
+{
+  Mixer* mixer = Mixer::getGlobalMasterMixer();
+  if ( mixer == 0 ) return; // e.g. when no soundcard is available
+  MixDevice *md = Mixer::getGlobalMasterMD();
+  if ( md == 0 ) return; // shouldn't happen, but lets play safe
+  mixer->toggleMute(md->id()); 
+  showVolumeDisplay();
+}
+
 void KMixWindow::quit()
 {
 //     kDebug(67100) << "quit";
@@ -635,7 +731,7 @@ void KMixWindow::applyPrefs( KMixPrefDlg *prefDlg )
    }
 
    if ( labelsHasChanged || ticksHasChanged || dockwidgetHasChanged || toplevelOrientationHasChanged || systrayPopupHasChanged) {
-      recreateGUI();
+      recreateGUI(true);
    }
 
    this->repaint(); // make KMix look fast (saveConfig() often uses several seconds)
@@ -683,6 +779,12 @@ void KMixWindow::slotConfigureCurrentView()
     if (view) view->configureView();
 }
 
+void KMixWindow::slotSelectMaster()
+{
+   DialogSelectMaster* dsm = new DialogSelectMaster(Mixer::getGlobalMasterMixer());
+   if (dsm) dsm->show();
+}
+
 void KMixWindow::newMixerShown(int /*tabIndex*/ ) {
    KMixerWidget* mw = (KMixerWidget*)m_wsMixers->currentWidget();
    if (mw) {
